# -*- coding: utf-8 -*-
import collections
import os
import requests
import yaml


CACHE_SETTINGS = {
    'attribute': 'pyzanata.requests.session',
    'cache_object': None,
}
RESTAPI_SPEC = {}

zanatarestapi = os.path.join(os.path.dirname(__file__), 'restapi.yaml')

with open(zanatarestapi, 'r') as zrapi:
    RESTAPI_SPEC = yaml.load(zrapi)

ZanataCredentials = collections.namedtuple('Connection', 'url user token')


class ZanataMethod(object):

    def __init__(self, endpoint, method, spec):
        self.endpoint = endpoint
        self.method = method
        self.spec = spec

    @property
    def _credentials(self):
        return self.endpoint.resource.client.credentials

    def _path(self, **kwargs):
        return self.spec['endpoint'].format(**kwargs)

    def _url(self, **kwargs):
        return self._credentials.url + self._path(**kwargs)

    @property
    def _headers(self):
        return {'Accept': self.spec['methods'][self.method]}

    @property
    def _session(self):
        """a requests session by zanata user
        """
        # needs evaluation if this can be handled more elegant
        if CACHE_SETTINGS['cache_object'] is not None:
            cache_object = CACHE_SETTINGS['cache_object']
            session_cache = getattr(
                cache_object,
                CACHE_SETTINGS['attribute'],
                None
            )
            if session_cache is None:
                session_cache = dict()
                setattr(
                    cache_object,
                    CACHE_SETTINGS['attribute'],
                    session_cache
                )
        else:
            session_cache = dict()
        session = session_cache.get(self._credentials, None)
        if session is None:
            session = requests.Session()
            session.headers.update({
                'X-Auth-User': self._credentials.user,
                'X-Auth-Token': self._credentials.token,
            })
            session_cache[self._credentials] = session
        return session

    def __call__(self, params=None, data=None, **kwargs):
        req_method = getattr(self._session, self.method.lower())
        resp = req_method(
            self._url(**kwargs),
            data=data,
            params=params,
            headers=self._headers
        )
        return resp


class ZanataEndpoint(object):

    def __init__(self, resource, name, spec):
        self.resource = resource
        self.name = name
        self.spec = spec

    def __getattr__(self, attribute):
        if attribute in self.spec['methods']:
            return ZanataMethod(
                self,
                attribute,
                self.spec
            )
        raise AttributeError(
            'Attribute {0} is not a valid method (nor a usal attribute).\n'
            'valid are:\n'.format(
                attribute,
                '\n'.join(self.spec['methods'].keys())
            )
        )


class ZanataResource(object):

    def __init__(self, client, name, spec):
        self.client = client
        self.name = name
        self.spec = spec

    def __getattr__(self, attribute):
        if attribute in self.spec:
            return ZanataEndpoint(self, attribute, self.spec[attribute])
        raise AttributeError(
            'Attribute {0} is not a valid endpoint (nor a usal attribute).\n'
            'valid are:\n'.format(
                attribute,
                '\n'.join(self.spec.keys())
            )
        )


class ZanataClient(object):

    def __init__(self, credentials):
        """
        credentials - a ZanataCredentials named tuple
        """
        self.credentials = credentials

    def __getattr__(self, attribute):
        if attribute in RESTAPI_SPEC:
            return ZanataResource(self, attribute, RESTAPI_SPEC[attribute])
        raise AttributeError(
            'Attribute {0} is not a valid Resource (nor a usal attribute).'
        )



# -*- coding: utf-8 -*-
import responses
import unittest


class TestZanataClient(unittest.TestCase):

    def test_structure(self):
        from pyzanata import ZanataCredentials
        from pyzanata import ZanataClient
        from pyzanata import ZanataResource
        from pyzanata import ZanataEndpoint
        from pyzanata import ZanataMethod
        credentials = ZanataCredentials(
            'https://foo.bar/api',
            'user',
            'secret'
        )
        zc = ZanataClient(credentials)
        zr = zc.AccountResource
        self.assertTrue(isinstance(zr, ZanataResource))
        ze = zr.accounts
        self.assertTrue(isinstance(ze, ZanataEndpoint))
        zm = ze.PUT
        self.assertTrue(isinstance(zm, ZanataMethod))
        self.assertTrue(callable(zm))

    def test_method(self):
        from pyzanata import ZanataCredentials
        from pyzanata import ZanataClient
        credentials = ZanataCredentials(
            'https://foo.bar/api',
            'foobarbaz',
            'secret'
        )
        zc = ZanataClient(credentials)
        zm = zc.AccountResource.accounts.GET
        self.assertEqual(
            zm._path(username='foobarbaz'),
            '/accounts/u/foobarbaz'
        )
        self.assertEqual(
            zm._url(username='foobarbaz'),
            'https://foo.bar/api/accounts/u/foobarbaz'
        )
        self.assertEqual(
            zm._headers,
            {'Accept': 'application/vnd.zanata.account+json'}
        )

    @responses.activate
    def test_method_call(self):
        expected_resp_json = [{
            u'defaultType': u'Podir',
            u'id': u'my-test',
            u'links': [{
                u'href': u'p/my-test',
                u'rel': u'self',
                u'type': u'application/vnd.zanata.project+json'
            }],
            u'name': u'MyWebsites Test',
            u'status': u'ACTIVE',
        }]
        responses.add(
            responses.GET,
            'https://foo.bar/api/projects',
            json=expected_resp_json,
            status=200
        )
        from pyzanata import ZanataCredentials
        from pyzanata import ZanataClient
        credentials = ZanataCredentials(
            'https://foo.bar/api',
            'foobarbaz',
            'secret'
        )
        zc = ZanataClient(credentials)
        resp = zc.ProjectsResource.projects.GET()
        got_resp = resp.json()
        self.assertDictEqual(got_resp[0], expected_resp_json[0])


def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestZanataClient))
    return suite


if __name__ == '__main__':
    runner = unittest.TextTestRunner(failfast=True)
    runner.run(suite())



# -*- coding: utf-8 -*-
"""Installer for the example.package package."""
from setuptools import setup

setup(
    # thanks to this bug
    # https://github.com/pypa/setuptools/issues/1136
    # we need one line in here:
    package_dir={'': 'src'},
)
